import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class particleBank
{
	final static int MAX_PARTICLES = 512;
	final static int MIN_CLUSTER_PARTICLES = 4;
	final static float DEFAULT_CLUSTER_PARTICLE_SPEED = 0.4f;

	particle[] bParticle;
	int numParticles;
	int maxParticles;

        //spawn a random cluster of particles of, at most, maxNum particles, and at most maxRad radius
	//alphaFade tells which direction, if any, the alpha will increment or decrement on each logic cycle (-1 : 0 : 1 are good values..)
	public void spawnRectFilledParticleCluster(vertex coords, Color col, int alphaFade, int maxRad, int maxNum)
	{
        	int randNum = (int)(Math.random()*(maxNum-MIN_CLUSTER_PARTICLES))+MIN_CLUSTER_PARTICLES;
                
		for(int k = 0; k < randNum; k++){
                	float r = (float)Math.random()*maxRad;
                        addParticle(coords, new vertex(getRandomNegativity()*(float)Math.random()*DEFAULT_CLUSTER_PARTICLE_SPEED, getRandomNegativity()*(float)Math.random()*DEFAULT_CLUSTER_PARTICLE_SPEED), col, r, r, particle.SHAPE_RECT_FILLED, (int)(Math.random()*100), alphaFade);
                        }
	}

	int getRandomNegativity(){ return (Math.random() < 0.5 ? -1 : 1); }

	//gets first open particle slot
	int getOpenSlot()
	{
		for(int i = 0; i < maxParticles; i++){
			if(bParticle[i] == null) return i;
			if(bParticle[i].isInactive()) return i;
			}
		return -1;
	}

	//add a particle to the bank..
	public void addParticle(vertex coords, vertex vCoords, Color col, float hR, float vR, int sh, int clife)
	{
		int slot = getOpenSlot();
		if(slot == -1) return;

		if(slot >= numParticles) numParticles = slot+1;
		bParticle[slot] = new particle(coords, vCoords, col, hR, vR, sh, clife, 0);
	}
	public void addParticle(vertex coords, vertex vCoords, Color col, int iCol, float hR, float vR, int sh, int clife)
	{
		int slot = getOpenSlot();
		if(slot == -1) return;

		if(slot >= numParticles) numParticles = slot+1;
		bParticle[slot] = new particle(coords, vCoords, col, iCol, hR, vR, sh, clife, 0);
	}
	public void addParticle(vertex coords, vertex vCoords, String str, Font ft, Color col, float hR, float vR, int sh, int clife)
	{
		int slot = getOpenSlot();
		if(slot == -1) return;

		if(slot >= numParticles) numParticles = slot+1;
		bParticle[slot] = new particle(coords, vCoords, str, ft, col, hR, vR, sh, clife, 0);
	}
	//these have the alpha fade value passed to them, instead of defaulting to 0..
	public void addParticle(vertex coords, vertex vCoords, Color col, float hR, float vR, int sh, int clife, int alphaFade)
	{
		int slot = getOpenSlot();
		if(slot == -1) return;

		if(slot >= numParticles) numParticles = slot+1;
		bParticle[slot] = new particle(coords, vCoords, col, hR, vR, sh, clife, alphaFade);
	}
	public void addParticle(vertex coords, vertex vCoords, Color col, int iCol, float hR, float vR, int sh, int clife, int alphaFade)
	{
		int slot = getOpenSlot();
		if(slot == -1) return;

		if(slot >= numParticles) numParticles = slot+1;
		bParticle[slot] = new particle(coords, vCoords, col, iCol, hR, vR, sh, clife, alphaFade);
	}
	public void addParticle(vertex coords, vertex vCoords, String str, Font ft, Color col, float hR, float vR, int sh, int clife, int alphaFade)
	{
		int slot = getOpenSlot();
		if(slot == -1) return;

		if(slot >= numParticles) numParticles = slot+1;
		bParticle[slot] = new particle(coords, vCoords, str, ft, col, hR, vR, sh, clife, alphaFade);
	}
	
	//disable a particle in the bank with this string in it...
	public void disableParticleWithString(String str)
	{
		for(int i = 0; i < numParticles; i++)
		{
			if(bParticle[i] == null || !bParticle[i].isActive()) continue;
			if(bParticle[i].txt.equals(str))
			{
				bParticle[i].kill();
				return;
			}
		}
	}	

	void setDefaults()
	{
		numParticles = 0;
	}

	//allocate memory for particle array
	void initializeParticleArray(int num)
	{
		maxParticles = num;
		bParticle = new particle[maxParticles];
	}

	//constructors
	public particleBank()
	{
		setDefaults();
		initializeParticleArray(MAX_PARTICLES);
	}
	public particleBank(int num)
	{
		setDefaults();
		initializeParticleArray(num);
	}
}
