class player
{
        final static int MAX_WORDS = 128;
        final static int MAX_WORD_LENGTH = 64;
        final static int DIR_NONE = -1;
        final static int DIR_UP = 0;
        final static int DIR_UPRIGHT = 1;
        final static int DIR_RIGHT = 2;
        final static int DIR_DOWNRIGHT = 3;
        final static int DIR_DOWN = 4;
        final static int DIR_DOWNLEFT = 5;
        final static int DIR_LEFT = 6;
        final static int DIR_UPLEFT = 7;

        String name = null;       //current name associated with this player
        int mode;                 //current state
        String[] words = new String[MAX_WORDS];     //current word accomplished list
        char[] charAcc = new char[MAX_WORD_LENGTH]; //current character accumulator when dragging a line
        int numWords;             //# words finished
        int startX, startY;       //holds positions from when the mouse was first pressed
        int dx, dy;               //current direction of the mouse drag
        int endX, endY;           //position of the mouse on release
        int rowStart, colStart;   //row,col starting positions
        int rowStart2, colStart2; //secondary starting positions
        int rowEnd, colEnd;       //row,col release positions
        int drawMode;             //current line drawing mode
        int selDir;               //current selection direction

        //constructors..
        public player()
        {
                setDefault();
        }
        public player(String str)
        {
                name = str;
                setDefault();
        }

        //set default values
        public void setDefault()
        {
                mode = 0;
                drawMode = -1;
                numWords = 0;
                startX = startY = dx = dy = endX = endY = rowStart = colStart = rowStart2 = colStart2 = rowEnd = colEnd = 0;
                selDir = DIR_NONE;
                clearAccumulator();
        }

        //clear the character accumulator
        public void clearAccumulator()
        {
                for(int i = 0; i < MAX_WORD_LENGTH; i++) charAcc[i] = '\0';
        }

        //set the x/y slopes for the selection
        public void setSelectionSlope()
        {
                if(colEnd-colStart < 0) dx = -1;
                else if(colEnd-colStart > 0) dx = 1;
                else dx = 0;

                if(rowEnd-rowStart < 0) dy = -1;
                else if(rowEnd-rowStart > 0) dy = 1;
                else dy = 0;

                if(dx < 0){
                        if(dy == 0) selDir = DIR_LEFT;
                        else if(dy < 0){
                                if(rowEnd-rowStart == colEnd-colStart){
                                        selDir = DIR_UPLEFT;
                                        }
                                else selDir = DIR_NONE;
                                }
                        else if(dy > 0){
                                if(Math.abs(rowEnd-rowStart) == Math.abs(colEnd-colStart)){
                                        selDir = DIR_DOWNLEFT;
                                        }
                                else{
                                        selDir = DIR_NONE;
                                        }
                                }
                        }
                else if(dx > 0){
                        if(dy == 0) selDir = DIR_RIGHT;
                        else if(dy < 0){
                                if(Math.abs(rowEnd-rowStart) == Math.abs(colEnd-colStart)){
                                        selDir = DIR_UPRIGHT;
                                        }
                                else selDir = DIR_NONE;
                                }
                        else if(dy > 0){
                                if(rowEnd-rowStart == colEnd-colStart){
                                        selDir = DIR_DOWNRIGHT;
                                        }
                                else selDir = DIR_NONE;
                                }
                        }
                else{
                        if(dy == 0) selDir = DIR_NONE;
                        else if(dy < 0){
                                selDir = DIR_UP;
                                }
                        else if(dy > 0){
                                selDir = DIR_DOWN;
                                }
                        }
        }

        //gets the selection string after mouse release
        public String getSelectionString(puzzle pz)
        {
                String tempStr = "";
                if(hasDirection()){
                        int c = colStart;
                        int r = rowStart;
                        //while(c != colEnd || r != rowEnd){
                        while((c != colEnd || r != rowEnd) && c < pz.numCols && r < pz.numRows){
                                tempStr += pz.pTile[r][c].aChar;
                                c += dx;
                                r += dy;
                                }
                        if(r < pz.numRows && c < pz.numCols) tempStr += pz.pTile[r][c].aChar;
                        }
                return tempStr;
        }

        //marks the selection string after mouse release
        public void markSelectionString(puzzle pz)
        {
                String tempStr = "";
                if(hasDirection()){
                        int c = colStart;
                        int r = rowStart;
                        //while(c != colEnd || r != rowEnd){
                        while((c != colEnd || r != rowEnd) && c < pz.numCols && r < pz.numRows){
                                pz.pTile[r][c].crossOut();
                                c += dx;
                                r += dy;
                                }
                        pz.pTile[r][c].crossOut();
                        }
        }

        //sets drawing to true
        public void setDrawOn()
        {
                drawMode = 0;
        }

        //sets drawing to false
        public void setDrawOff()
        {
                drawMode = -1;
                selDir = DIR_NONE;
                rowEnd = rowStart;
                colEnd = colStart;
        }

        //tells if a direction is set currently
        public boolean hasDirection()
        {
                return (selDir != DIR_NONE);
        }

        //tells if we're drawing or not
        public boolean notDrawing()
        {
                return (drawMode == -1);
        }

        //tells if we're drawing or not
        public boolean isDrawing()
        {
                return (drawMode == 0);
        }
}
