import java.awt.*;
import java.awt.image.*;
import java.util.*;

//defines a word puzzle..
class puzzle
{
        final static int MAX_ROWS = 64;
        final static int MAX_COLS = 64;
        final static int MAX_WORDS = 128;
        final static int MIN_WORD_LENGTH = 4;
        final static int DEFAULT_TILE_W = 20; //16;
        final static int DEFAULT_TILE_H = DEFAULT_TILE_W;
        final static int DEFAULT_WORD_ROWS = 12;
        final static int DEFAULT_WORD_COLS = 6;
        final static float DEFAULT_COL_ROW_RATIO = 1.25f;   //col dimensions are 60% greater than row dimensions..
        final static float DEFAULT_NON_TO_WORD_RATIO = 0.45f; //ratio of tiles that don't contain words to those that do..
        final static int MAX_LEFTOVER_WORDS = 64;

        final static int DIR_NONE = -1;
        final static int DIR_UP = 0;
        final static int DIR_UPRIGHT = 1;
        final static int DIR_RIGHT = 2;
        final static int DIR_DOWNRIGHT = 3;
        final static int DIR_DOWN = 4;
        final static int DIR_DOWNLEFT = 5;
        final static int DIR_LEFT = 6;
        final static int DIR_UPLEFT = 7;

        int numRows;      //puzzle dimensions
        int numCols;
        int numWords;     //# words to find..
        int numWordsStruck;  //# words completed thusfar
        int numWordRows;  //# word rows in the listing
        int numWordCols;  //# word cols in the listing
        int longestWordLength;   //longest word length on this puzzle
        String title;     //title of the puzzle
        String[] leftoverWord = new String[MAX_LEFTOVER_WORDS];   //leftover words
        vertex pos,vel;   //position, velocity vectors
        vertex listPos;   //position of the list
        vertex boardPos;  //position of the board
        vertex resetPos;  //position of "start new game" button
        vertex capsPos;   //position of "toggle caps" button
        vertex debugPos;  //position of the "debug" button
        wFont font;       //font used on this board
        wFont listFont;   //font used on the word list
        Color textColor;  //color of unaltered text
        Color textFoundColor; //color of found words on board
        Color crossColor; //color of crossed-out words
        Color bgColor;    //color of the puzzle background surface
        Color borderColor; //color of the board border
        Color hlColor;    //color of word highlighting
        int tileW, tileH; //tile dimensions
        tile[][] pTile = new tile[MAX_ROWS][MAX_COLS];  //the letter tiles
        word[] pWord = new word[MAX_WORDS];             //the word bank
        vertex[] wordCrossStart = new vertex[MAX_WORDS]; //positions for the cross out on the board
        vertex[] wordCrossEnd = new vertex[MAX_WORDS];

        boolean debugOn;          //tells if debug is enabled

        boolean mouseCapsOver;    //tells if the mouse is currently over the "toggle caps" button
        boolean mouseResetOver;   //tells if the mouse is currently over the "reset" button
        boolean mouseDebugOver;   //tells if the mouse is currently over the "debug" button

        //constructors
        public puzzle()
        {
                pos = new vertex();
                vel = new vertex(0.0f, 0.0f, 0.0f);
                font = null;
                setDefault();
        }
        public puzzle(String str, vertex coords, int nRows, int nCols)
        {
                pos = new vertex(coords.x, coords.y, coords.z);
                vel = new vertex(0.0f, 0.0f, 0.0f);
                font = null;
                listFont = null;
                numRows = nRows;
                numCols = nCols;
                numWords = 0;
                setDefault();
        }
        public puzzle(String str, vertex coords, String words[], int nW)
        {
                pos = new vertex(coords.x, coords.y, coords.z);
                vel = new vertex(0.0f, 0.0f, 0.0f);
                font = null;
                listFont = null;
                numWords = 0;
                setDefault();
                addWordList(words, nW);
                resizePuzzleDimensions();
        }

        //set some default values
        public void setDefault()
        {
                textColor = Color.black;
                bgColor = Color.white;
                crossColor = Color.red;
                textFoundColor = new Color(0.0f, 0.4f, 0.0f);
                hlColor = new Color(0.5f, 0.5f, 0.5f);
                borderColor = new Color(157, 185, 255);
                tileW = DEFAULT_TILE_W;
                tileH = DEFAULT_TILE_H;
                numWordRows = DEFAULT_WORD_ROWS;
                numWordCols = DEFAULT_WORD_COLS;
                numWordsStruck = 0;
                longestWordLength = 0;
                mouseCapsOver = mouseResetOver = mouseDebugOver = debugOn = false;
        }

        //set the board characters up..
        public void setBoardCharacters()
        {
                //set all tiles as random for now..
                for(int r = 0; r < numRows; r++){
                        for(int c = 0; c < numCols; c++){
                                pTile[r][c] = new tile();
                                pTile[r][c].setRandomChar();
                                }
                        }

                //make a temp array and sort by length (best to place big words first..)
                String[] tempStr = new String[numWords];
                for(int i = 0; i < numWords; i++){
                        tempStr[i] = pWord[i].txt;
                        }
                //use built-in java sorter (use length comparator)
                java.util.Arrays.sort(tempStr, new stringLengthComparator(-1));

                //start placing words..
                boolean wordDone = false;
                int expand = 0;     //use this to expand the board if we get stuck
                for(int w = 0; w < numWords; w++){
                        wordDone = false;
                        int randDir = -1;
                        int startingDir = -1;

                        //System.out.println("Placing " + tempStr[w]);
                        //get the current time so we can restart the loop
                        //if we get stuck calculating in here too long
                        long curMs = System.currentTimeMillis();
                        while(!wordDone){
                                //restart the loop if we're on one word more than .5 seconds
                                if(System.currentTimeMillis()-curMs >= 500){
                                        if(expand == 0){
                                                numRows++;
                                                expand = 1;
                                                }
                                        else{
                                                numCols++;
                                                expand = 0;
                                                }
                                        //w = 0;
                                        //break;

                                        //restart the curms counter
                                        curMs = System.currentTimeMillis();
                                        //goto the bottom of the while loop and try again
                                        continue;
                                        }

                                int r = (int)(Math.random()*numRows);
                                int c = (int)(Math.random()*numCols);
                                int dx = 0, dy = 0;

                                //we're just starting a new check.. get a random row,col and a new random direction
                                if(randDir == -1){
                                        startingDir = randDir = (int)(Math.random()*8.0);   //try any one of the 8 directions first
                                        c = (int)(Math.random()*numCols);
                                        r = (int)(Math.random()*numRows);

                                        if(pTile[r][c] == null) pTile[r][c] = new tile((char)((int)(Math.random()*26)+(int)('a')), c, r);

                                        while(pTile[r][c].wordHere && pTile[r][c].aChar != tempStr[w].charAt(0)){
                                                c = (int)(Math.random()*numCols);
                                                r = (int)(Math.random()*numRows);
                                                if(pTile[r][c] == null) pTile[r][c] = new tile((char)((int)(Math.random()*26)+(int)('a')), c, r);
                                                //System.out.println("r,c:(" + r + "," + c + ")");
                                                //System.out.print("while(");
                                                //System.out.print("" + pTile[r][c].wordHere);
                                                //System.out.print(" && " + pTile[r][c].aChar);
                                                //System.out.println(" != " + tempStr[w].charAt(0) + ")");
                                                }
                                        }

                                switch(randDir){
                                        case DIR_UP:
                                                dx = 0; dy = -1;
                                                break;
                                        case DIR_UPRIGHT:
                                                dx = 1; dy = -1;
                                                break;
                                        case DIR_RIGHT:
                                                dx = 1; dy = 0;
                                                break;
                                        case DIR_DOWNRIGHT:
                                                dx = 1; dy = 1;
                                                break;
                                        case DIR_DOWN:
                                                dx = 0; dy = 1;
                                                break;
                                        case DIR_DOWNLEFT:
                                                dx = -1; dy = 1;
                                                break;
                                        case DIR_LEFT:
                                                dx = -1; dy = 0;
                                                break;
                                        case DIR_UPLEFT:
                                                dx = -1; dy = -1;
                                                break;
                                        case -1:
                                                continue;
                                                //break;
                                        default:
                                                break;
                                        }

                                boolean gotIt = false;
                                for(int l = 0; !gotIt && l < tempStr[w].length(); l++){
                                //for(int l = 0; !gotIt && l < pWord[w].length(); l++){
                                        if(r+l*dy < 0 || r+l*dy >= numRows || c+l*dx < 0 || c+l*dx >= numCols || pTile[r+l*dy][c+l*dx] == null){
                                                gotIt = true;
                                                continue;
                                                }
                                        if(pTile[r+l*dy][c+l*dx].wordHere){
                                        //if(pTile[r+l*dy][c+l*dx] != null && pTile[r+l*dy][c+l*dx].wordHere){
                                                //if(pTile[r+l*dy][c+l*dx].aChar != pWord[w].charAt(l)){
                                                if(pTile[r+l*dy][c+l*dx].aChar != tempStr[w].charAt(l)){
                                                        gotIt = true;
                                                        }
                                                }
                                        }
                                if(!gotIt){
                                        System.out.println("placing '" + pWord[w].txt + "' at (" + r + ", " + c + ")");
                                        //for(int l = 0; l < pWord[w].length(); l++){
                                        for(int l = 0; l < tempStr[w].length(); l++){
                                                pTile[r+l*dy][c+l*dx] = new tile();
                                                //pTile[r+l*dy][c+l*dx].setChar(pWord[w].charAt(l));
                                                pTile[r+l*dy][c+l*dx].setChar(tempStr[w].charAt(l));
                                                }
                                        //store these values so we can know the word placement and direction on the board
                                        pTile[r][c].wordStart = true;
                                        pTile[r][c].wordLength = tempStr[w].length();
                                        pTile[r][c].wordDx = dx;
                                        pTile[r][c].wordDy = dy;
                                        wordDone = true;
                                        }
                                else{
                                        randDir = (randDir+1)%8;  //go counter-clockwise to check the next direction
                                        if(randDir == startingDir) randDir = -1;
                                        }
                                }
                        //System.out.println("Word '" + pWord[w].txt + "' placed successfully..");
                        }
                //place leftover letters

                //make sure all tiles are non-null
                for(int r = 0; r < numRows; r++){
                        for(int c = 0; c < numCols; c++){
                                if(pTile[r][c] == null){
                                        pTile[r][c] = new tile();
                                        pTile[r][c].setRandomChar();
                                        }
                                }
                        }
        }

        //sort the word list alphabetically
        public void sortWordListAlphabetically()
        {
                String[] tempStr = new String[numWords];
                for(int i = 0; i < numWords; i++){
                        //put this word's characters in the temp string
                        tempStr[i] = pWord[i].txt;

                        //make all words lower-case
                        tempStr[i] = tempStr[i].toLowerCase();
                        }

                //use java's sorter on the temp string array..
                java.util.Arrays.sort(tempStr);

                //move the strings back to the word array
                for(int i = 0; i < numWords; i++){
                        pWord[i].txt = tempStr[i];
                        }
        }

        //make the puzzle's dimensions 25% larger than the longest word
        public void resizePuzzleDimensions()
        {
                longestWordLength = 0;
                for(int i = 0; i < numWords; i++){
                        if(pWord[i].length() > longestWordLength) longestWordLength = pWord[i].length();
                        }
                //numRows = getBoardStartRows();
                //numCols = getBoardStartCols();
                numRows = longestWordLength+longestWordLength/3;
                numCols = (longestWordLength+2)*2;
        }

        //set the various positions
        public void setPositions(vertex coords, vertex boardCoords, int appW, int appH)
        {
                pos = new vertex(coords.x, coords.y, coords.z);
                boardPos = new vertex(boardCoords.x, boardCoords.y, boardCoords.z);
                //listPos = new vertex(wordCoords.x, wordCoords.y, wordCoords.z);
                //determine where to place the words based on the board position
                int lx = (int)boardPos.x-5;
                int ly = (int)boardPos.y+(numRows+1)*(tileH+1)-7;
                listPos = new vertex(lx, ly);
                resetPos = new vertex(appW-120, (int)boardPos.y+30);
                capsPos = new vertex(appW-160, (int)boardPos.y+60);
                debugPos = new vertex(appW-170, (int)boardPos.y+90);
        }

        //add an array of strings to the word list
        public void addWordList(String words[], int nW)
        {
                for(int i = 0; i < nW; i++){
                        addWord(words[i]);
                        }
        }

        //add a single word to the word list
        public void addWord(String str)
        {
                pWord[numWords++] = new word(str, -1);
        }

        //set the puzzle font stuff up..
        public void setFont(Graphics g, Font ft, int fW, int fH)
        {
                font = new wFont(g, ft, fW, fH);
                listFont = new wFont(g, ft, 3*fW/4+1, 3*fH/4+1);
        }

        //load a puzzle from a saved file
        public void loadFile(String filename)
        {
                remoteStream tempStream = new remoteStream("www.t3-i.com", "");
                int arraySize = MAX_WORDS;
                String[] tempStr = new String[arraySize];

                System.out.println("loading data from remote server..");
                tempStr = tempStream.loadData(filename, arraySize);
                int count = 0;

                //line 1 is the puzzle title
                title = tempStr[count++];

                //start reading the word list
                while(tempStr[count] != null){
                        System.out.println("adding '" + tempStr[count] + "'");
                        addWord(tempStr[count++]);
                        }

                //reset the "cross out" positions
                resetWordCrossouts();

                //sort the word list..
                sortWordListAlphabetically();

                //resize the puzzle dimensions accordingly
                resizePuzzleDimensions();
                setBoardCharacters();
        }

        //load a puzzle from a dictionary object
        public void loadFromDictionary(dict di, int maxLen, int maxWords)
        {
                int arraySize = maxWords;
                String[] tempStr = new String[arraySize];

                System.out.println("loading words from dictionary..");
                int count = 0;

                //set the puzzle title
                title = "randomly generated wordMix puzzle";

                //grab random words from the dictionary..
                while(count < maxWords){
                        tempStr[count] = di.getRandomWord(MIN_WORD_LENGTH, maxLen);
                        //if we get a duplicate word, keep getting a random word till we get a non-dupe
                        while(hasWord(tempStr[count]) || wordHasNonCharacters(tempStr[count])){
                                tempStr[count] = di.getRandomWord(MIN_WORD_LENGTH, maxLen);
                                }
                        System.out.println("adding '" + tempStr[count] + "'");
                        addWord(tempStr[count++]);
                        }

                //reset the "cross out" positions
                resetWordCrossouts();

                //sort the word list..
                sortWordListAlphabetically();

                //resize the puzzle dimensions accordingly
                resizePuzzleDimensions();
                setBoardCharacters();
        }

        //reset the cross out positions
        void resetWordCrossouts()
        {
                wordCrossStart = new vertex[numWords];
                wordCrossEnd = new vertex[numWords];

                for(int i = 0; i < numWords; i++){
                        wordCrossStart[i] = new vertex(-200, -200);
                        wordCrossEnd[i] = new vertex(-200, -200);
                        }
        }

        //sets a dummy batch of words up
        public void loadDummy(int num)
        {
                title = "dummy wordlist";
                addWord("amalgam");
                addWord("artichoke");
                addWord("aardvark");
                addWord("bellow");
                addWord("bellicose");
                addWord("cadence");
                addWord("catty");
                addWord("decadent");
                addWord("daffy");
                addWord("earful");
                addWord("eagerness");
                addWord("fascimile");
                addWord("gigantic");
                addWord("humorous");

                addWord("idiosynchrocy");
                addWord("jumpy");
                addWord("keymaker");
                addWord("lopsided");
                addWord("menacing");
                addWord("neverending");
                addWord("ominous");
                addWord("perilous");
                addWord("query");
                addWord("radically");
                addWord("steadfast");
                addWord("theater");

                addWord("unquiet");
                addWord("vehemenent");
                addWord("walkway");
                addWord("xylophone");
                addWord("zebra");
                addWord("test");
                addWord("balderdash");
                addWord("scoff");
                addWord("sandals");
                addWord("mesmerize");
                addWord("salutation");
                addWord("crevass");
                addWord("onus");

                addWord("verification");
                addWord("visage");
                addWord("carry");
                addWord("simple");
                addWord("paragon");
                addWord("sample");
                addWord("carton");
                addWord("tarry");
                addWord("grief");
                addWord("growl");
                addWord("sane");
                addWord("telephone");

                if(num > 0) numWords = num;

                //sort the word list..
                sortWordListAlphabetically();

                //resize the puzzle dimensions accordingly
                resizePuzzleDimensions();
                setBoardCharacters();
        }

        //set a new cross-out defined by these endpoints
        public void setNewCross(int x1, int y1, int x2, int y2)
        {
                wordCrossStart[numWordsStruck] = new vertex(x1, y1);
                wordCrossEnd[numWordsStruck] = new vertex(x2, y2);
        }

        //cross this word out
        public void crossOutWord(String str)
        {
                for(int i = 0; i < numWords; i++){
                        if(pWord[i].txt.equals(str)){
                                numWordsStruck++;
                                pWord[i].strike();
                                return;
                                }
                        }
        }

        //gets the average length of the words in the word list
        public float getAverageWordLength()
        {
                int sum = 0;
                for(int i = 0; i < numWords; i++) sum += pWord[i].length();
                if(numWords > 0) return (float)sum/numWords;

                return -1.0f;
        }

        //gets the # tiles (non-empty) needed at minimum for the # of words currently available
        public int getBoardMinTilesNeeded()
        {
                return (int)getAverageWordLength()*numWords;
        }

        //gets a # of starting cols based on specified variables
        public int getBoardStartCols()
        {
                //gets a rough starting point (NxN board)
                int dim = (int)Math.sqrt(getBoardMinTilesNeeded()+(int)(getBoardMinTilesNeeded()*DEFAULT_NON_TO_WORD_RATIO));
                return (int)(dim*DEFAULT_COL_ROW_RATIO);
        }

        //gets a # of starting rows based on specified variables
        public int getBoardStartRows()
        {
                //gets a rough starting point (NxN board)
                int dim = (int)Math.sqrt(getBoardMinTilesNeeded()+(int)(getBoardMinTilesNeeded()*DEFAULT_NON_TO_WORD_RATIO));
                return (int)(dim/DEFAULT_COL_ROW_RATIO);
        }

        //tells if this point lies within the board area
        public boolean pointWithinBoard(int x, int y)
        {
                int tlX = (int)(pos.x+boardPos.x);
                int tlY = (int)(pos.y+boardPos.y);
                //int brX = tlX+(numCols-1)*tileW;
                //int brY = tlY+(numRows-1)*tileH;
                int brX = tlX+numCols*tileW;
                int brY = tlY+numRows*tileH;

                return (x >= tlX && y >= tlY && x <= brX && y <= brY);
        }

        //tells if this point lies within the 'toggle caps' button
        public boolean pointWithinCapsButton(int x, int y)
        {
                int tlX = (int)capsPos.x;
                int tlY = (int)capsPos.y-20;
                int brX = tlX+160;
                int brY = tlY+30;

                return (x >= tlX && y >= tlY && x <= brX && y <= brY);
        }

        //tells if this point lies within the 'restart' button
        public boolean pointWithinResetButton(int x, int y)
        {
                int tlX = (int)resetPos.x;
                int tlY = (int)resetPos.y-20;
                int brX = tlX+130;
                int brY = tlY+30;

                return (x >= tlX && y >= tlY && x <= brX && y <= brY);
        }

        //tells if this point lies within the 'toggle debug' button
        public boolean pointWithinDebugButton(int x, int y)
        {
                int tlX = (int)debugPos.x;
                int tlY = (int)debugPos.y-20;
                int brX = tlX+160;
                int brY = tlY+30;

                return (x >= tlX && y >= tlY && x <= brX && y <= brY);
        }

        //tells if this word has non-alphabetic characters (like hyphens..)
        public boolean wordHasNonCharacters(String str)
        {
                for(int i = 0; i < str.length(); i++){
                        if(str.charAt(i) == '-'){
                                return true;
                                }
                        }
                return false;
        }

        //tells if this word is in the word list (doesn't check activity)
        public boolean hasWord(String str)
        {
                for(int i = 0; i < numWords; i++){
                        if(pWord[i].txt.equals(str)){
                                return true;
                                }
                        }
                return false;
        }

        //tells if this string lies in the word list
        public boolean hasWordActive(String str)
        {
                for(int i = 0; i < numWords; i++){
                        if(pWord[i].txt.equals(str)){
                                return !pWord[i].isStruck();
                                }
                        }
                return false;
        }

        //turn the "mouse over caps" on/off
        public void setCapsOver(boolean state)
        {
                mouseCapsOver = state;
        }

        //turn the "mouse over reset" on/off
        public void setResetOver(boolean state)
        {
                mouseResetOver = state;
        }

        //turn the "mouse over debug" on/off
        public void setDebugOver(boolean state)
        {
                mouseDebugOver = state;
        }

        //toggle the debug mode
        public void setDebug(boolean state)
        {
                debugOn = state;
        }

        //tell if the mouse is over the caps button
        public boolean onCapsButton()
        {
                return mouseCapsOver;
        }

        //tell if the mouse is over the caps button
        public boolean onResetButton()
        {
                return mouseResetOver;
        }

        //tell if the mouse is over the debug button
        public boolean onDebugButton()
        {
                return mouseDebugOver;
        }

        //tells if all words are currently crossed out
        public boolean allWordsCrossed()
        {
                for(int i = 0; i < numWords; i++){
                        if(!pWord[i].isStruck()) return false;
                        }
                return true;
        }

        //display the board
        void drawBoard(Graphics g, ImageObserver im, boolean upperCase)
        {
                int curX = (int)(pos.x+boardPos.x);
                int curY = (int)(pos.y+boardPos.y);

                //draw the border
                g.setColor(borderColor);
                g.drawRect(curX, curY, numCols*tileW, numRows*tileH);

                //draw a shadow on the lower right
                g.setColor(new Color(0.8f, 0.8f, 0.8f));
                g.drawLine(curX+1, curY+numRows*tileH+1, curX+numCols*tileW+1, curY+numRows*tileH+1);
                g.drawLine(curX+numCols*tileW+1, curY+numRows*tileH+1, curX+numCols*tileW+1, curY+1);

                //fill the middle in
                g.setColor(bgColor);
                g.fillRect(curX+1, curY+1, numCols*tileW-2, numRows*tileH-2);

                //draw the letters
                g.setColor(textColor);
                g.setFont(font.jFont);
                for(int r = 0; r < numRows; r++){
                        for(int c = 0; c < numCols; c++){
                                //if(pTile[r][c].wordHere){
                                //        g.setColor(new Color(0.8f, 0.8f, 0.8f));
                                //        g.fillRect(curX+c*tileW, curY+r*tileH, tileW, tileH);
                                //        g.setColor(textColor);
                                //        }
                                if(!upperCase){
                                        if(pTile[r][c] != null){
                                              g.drawString("" + pTile[r][c].aChar, curX+c*tileW+3, curY+(r+1)*tileH-4);
                                              }
                                        }
                                else{
                                        if(pTile[r][c] != null){
                                              if(debugOn && pTile[r][c].wordHere){
                                                    g.setColor(Color.green);
                                                    g.fillRect(curX+c*tileW, curY+r*tileH, tileW, tileH);
                                                    }
                                              //change text color depending on whether the word was found or not
                                              g.setColor(textColor);
                                              if(pTile[r][c].wordHere && pTile[r][c].isCrossed()) g.setColor(textFoundColor);

                                              g.drawString(("" + pTile[r][c].aChar).toUpperCase(), curX+c*tileW+3, curY+(r+1)*tileH-4);
                                              }
                                        }
                                }
                        }
        }

        //display the word crosses
        void drawCrossedWords(Graphics g, ImageObserver im)
        {
                int curX = (int)(pos.x+boardPos.x);
                int curY = (int)(pos.y+boardPos.y);
                //g.setColor(hlColor);
                g.setColor(new Color(0.0f, 0.0f, 0.0f, 0.5f));

                for(int w = 0; w < numWordsStruck; w++){
                        g.drawLine(curX+(int)wordCrossStart[w].x, curY+(int)wordCrossStart[w].y, curX+(int)wordCrossEnd[w].x, curY+(int)wordCrossEnd[w].y);
                        }
        }

        //display the word list
        void drawWordList(Graphics g, ImageObserver im, int appW, int appH, boolean upperCase)
        {
                g.setFont(listFont.jFont);
                g.setColor(textColor);
                int curX = 0;
                int curY = 0;
                for(int i = 0; i < numWords; i++){
                        //int modI = i%numWordRows;
                        //curY = listFont.h*modI;
                        //start in the next column if we go over the word limit for this column
                        //if(i > 0 && modI == 0){
                        //        curX += (longestWordLength+2)*font.w;
                        //        }

                        //int modI = i%numWordRows;

                        //start in the next column if we go over the word limit for this column (or go past the lower screen edge)
                        //if(i > 0 && modI == 0){
                        //        curX += (longestWordLength+2)*font.w;
                        //        }

                        //set the color to normal text color for un-crossed words, cross color otherwise
                        if(!pWord[i].isStruck()) g.setColor(textColor);
                        else g.setColor(crossColor);

                        if(!upperCase){
                                g.drawString(pWord[i].txt, (int)(pos.x+listPos.x)+curX, (int)(pos.y+listPos.y)+curY);
                                }
                        else{
                                g.drawString(pWord[i].txt.toUpperCase(), (int)(pos.x+listPos.x)+curX, (int)(pos.y+listPos.y)+curY);
                                }
                        if(pWord[i].isStruck()){
                                g.drawLine((int)(pos.x+listPos.x)+curX, (int)(pos.y+listPos.y)+curY-font.h/4, (int)(pos.x+listPos.x)+curX+pWord[i].length()*font.w, (int)(pos.y+listPos.y)+curY-font.h/4);
                                }

                        curY += listFont.h;
                        if((int)(pos.y+listPos.y)+curY > appH){
                                curY = 0;
                                curX += (longestWordLength)*font.w;
                                }
                        }
        }

        //draw the onscreen buttons/options
        void drawButtons(Graphics g, ImageObserver im, boolean upperCase)
        {
                g.setFont(font.jFont.deriveFont(Font.PLAIN, 17));

                if(onResetButton()){   //highlight the selected button(s)
                        g.setColor(new Color(0.9f, 0.9f, 0.9f));
                        g.fillRect((int)(pos.x+resetPos.x), (int)(pos.y+resetPos.y-20), 120, 30);
                        }
                if(onCapsButton()){
                        g.setColor(new Color(0.9f, 0.9f, 0.9f));
                        g.fillRect((int)(pos.x+capsPos.x), (int)(pos.y+capsPos.y-20), 160, 30);
                        }
                if(onDebugButton()){
                        g.setColor(new Color(0.9f, 0.9f, 0.9f));
                        g.fillRect((int)(pos.x+debugPos.x), (int)(pos.y+debugPos.y-20), 170, 30);
                        }
                //draw the "reset" button
                g.setColor(Color.black);
                g.drawString("( restart )", (int)(pos.x+resetPos.x), (int)(pos.y+resetPos.y));

                //draw the "toggle caps" button
                if(!upperCase) g.drawString("( toggle caps )", (int)(pos.x+capsPos.x), (int)(pos.y+capsPos.y));
                else g.drawString("( TOGGLE CAPS )", (int)(pos.x+capsPos.x), (int)(pos.y+capsPos.y));

                //draw the "debug" button
                g.drawString("( toggle debug )", (int)(pos.x+debugPos.x), (int)(pos.y+debugPos.y));
        }

        //display the puzzle
        public void draw(Graphics g, ImageObserver im, int appW, int appH, boolean upperCase)
        {
                drawBoard(g, im, upperCase);
                drawWordList(g, im, appW, appH, upperCase);
                drawCrossedWords(g, im);
                drawButtons(g, im, upperCase);
        }
}
